/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GDM_DBUS_GLUE_H__
#define __GDM_DBUS_GLUE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.DBus.Peer */

#define GDM_DBUS_TYPE_PEER (gdm_dbus_peer_get_type ())
#define GDM_DBUS_PEER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_PEER, GdmDBusPeer))
#define GDM_DBUS_IS_PEER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_PEER))
#define GDM_DBUS_PEER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), GDM_DBUS_TYPE_PEER, GdmDBusPeerIface))

struct _GdmDBusPeer;
typedef struct _GdmDBusPeer GdmDBusPeer;
typedef struct _GdmDBusPeerIface GdmDBusPeerIface;

struct _GdmDBusPeerIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_get_machine_id) (
    GdmDBusPeer *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_ping) (
    GdmDBusPeer *object,
    GDBusMethodInvocation *invocation);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusPeer, g_object_unref)
#endif

GType gdm_dbus_peer_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *gdm_dbus_peer_interface_info (void);
guint gdm_dbus_peer_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void gdm_dbus_peer_complete_get_machine_id (
    GdmDBusPeer *object,
    GDBusMethodInvocation *invocation,
    const gchar *unnamed_arg0);

void gdm_dbus_peer_complete_ping (
    GdmDBusPeer *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void gdm_dbus_peer_call_get_machine_id (
    GdmDBusPeer *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_peer_call_get_machine_id_finish (
    GdmDBusPeer *proxy,
    gchar ** out_unnamed_arg0,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_peer_call_get_machine_id_sync (
    GdmDBusPeer *proxy,
    gchar ** out_unnamed_arg0,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_peer_call_ping (
    GdmDBusPeer *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_peer_call_ping_finish (
    GdmDBusPeer *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_peer_call_ping_sync (
    GdmDBusPeer *proxy,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define GDM_DBUS_TYPE_PEER_PROXY (gdm_dbus_peer_proxy_get_type ())
#define GDM_DBUS_PEER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_PEER_PROXY, GdmDBusPeerProxy))
#define GDM_DBUS_PEER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_PEER_PROXY, GdmDBusPeerProxyClass))
#define GDM_DBUS_PEER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_PEER_PROXY, GdmDBusPeerProxyClass))
#define GDM_DBUS_IS_PEER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_PEER_PROXY))
#define GDM_DBUS_IS_PEER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_PEER_PROXY))

typedef struct _GdmDBusPeerProxy GdmDBusPeerProxy;
typedef struct _GdmDBusPeerProxyClass GdmDBusPeerProxyClass;
typedef struct _GdmDBusPeerProxyPrivate GdmDBusPeerProxyPrivate;

struct _GdmDBusPeerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  GdmDBusPeerProxyPrivate *priv;
};

struct _GdmDBusPeerProxyClass
{
  GDBusProxyClass parent_class;
};

GType gdm_dbus_peer_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusPeerProxy, g_object_unref)
#endif

void gdm_dbus_peer_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusPeer *gdm_dbus_peer_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusPeer *gdm_dbus_peer_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void gdm_dbus_peer_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusPeer *gdm_dbus_peer_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusPeer *gdm_dbus_peer_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define GDM_DBUS_TYPE_PEER_SKELETON (gdm_dbus_peer_skeleton_get_type ())
#define GDM_DBUS_PEER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_PEER_SKELETON, GdmDBusPeerSkeleton))
#define GDM_DBUS_PEER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_PEER_SKELETON, GdmDBusPeerSkeletonClass))
#define GDM_DBUS_PEER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_PEER_SKELETON, GdmDBusPeerSkeletonClass))
#define GDM_DBUS_IS_PEER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_PEER_SKELETON))
#define GDM_DBUS_IS_PEER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_PEER_SKELETON))

typedef struct _GdmDBusPeerSkeleton GdmDBusPeerSkeleton;
typedef struct _GdmDBusPeerSkeletonClass GdmDBusPeerSkeletonClass;
typedef struct _GdmDBusPeerSkeletonPrivate GdmDBusPeerSkeletonPrivate;

struct _GdmDBusPeerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  GdmDBusPeerSkeletonPrivate *priv;
};

struct _GdmDBusPeerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType gdm_dbus_peer_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusPeerSkeleton, g_object_unref)
#endif

GdmDBusPeer *gdm_dbus_peer_skeleton_new (void);


G_END_DECLS

#endif /* __GDM_DBUS_GLUE_H__ */
