
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#ifndef GDM_SESSION_WORKER_ENUM_TYPES_H
#define GDM_SESSION_WORKER_ENUM_TYPES_H

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "/build/gdm/src/build/../gdm/daemon/gdm-session-worker.h" */
GType gdm_session_worker_state_get_type (void) G_GNUC_CONST;
#define GDM_TYPE_SESSION_WORKER_STATE (gdm_session_worker_state_get_type ())
G_END_DECLS

#endif /* GDM_SESSION_WORKER_ENUM_TYPES_H */

/* Generated data ends here */

