
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */


#include <glib-object.h>
#include "/build/gdm/src/build/../gdm/daemon/gdm-session-worker.h"
/* enumerations from "/build/gdm/src/build/../gdm/daemon/gdm-session-worker.h" */
GType gdm_session_worker_state_get_type (void) G_GNUC_CONST;

GType
gdm_session_worker_state_get_type (void)
{
        static GType etype = 0;

        if (G_UNLIKELY(etype == 0)) {
                static const GEnumValue values[] = {
                { GDM_SESSION_WORKER_STATE_NONE, "GDM_SESSION_WORKER_STATE_NONE", "none" },
                { GDM_SESSION_WORKER_STATE_SETUP_COMPLETE, "GDM_SESSION_WORKER_STATE_SETUP_COMPLETE", "setup-complete" },
                { GDM_SESSION_WORKER_STATE_AUTHENTICATED, "GDM_SESSION_WORKER_STATE_AUTHENTICATED", "authenticated" },
                { GDM_SESSION_WORKER_STATE_AUTHORIZED, "GDM_SESSION_WORKER_STATE_AUTHORIZED", "authorized" },
                { GDM_SESSION_WORKER_STATE_ACCREDITED, "GDM_SESSION_WORKER_STATE_ACCREDITED", "accredited" },
                { GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_SAVED, "GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_SAVED", "account-details-saved" },
                { GDM_SESSION_WORKER_STATE_SESSION_OPENED, "GDM_SESSION_WORKER_STATE_SESSION_OPENED", "session-opened" },
                { GDM_SESSION_WORKER_STATE_SESSION_STARTED, "GDM_SESSION_WORKER_STATE_SESSION_STARTED", "session-started" },
                { 0, NULL, NULL }
        };

        etype = g_enum_register_static (g_intern_static_string ("GdmSessionWorkerState"), values);
    }

    return etype;
}
 /**/

/* Generated data ends here */

