/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GDM_DISPLAY_GLUE_H__
#define __GDM_DISPLAY_GLUE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.DisplayManager.Display */

#define GDM_DBUS_TYPE_DISPLAY (gdm_dbus_display_get_type ())
#define GDM_DBUS_DISPLAY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_DISPLAY, GdmDBusDisplay))
#define GDM_DBUS_IS_DISPLAY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_DISPLAY))
#define GDM_DBUS_DISPLAY_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), GDM_DBUS_TYPE_DISPLAY, GdmDBusDisplayIface))

struct _GdmDBusDisplay;
typedef struct _GdmDBusDisplay GdmDBusDisplay;
typedef struct _GdmDBusDisplayIface GdmDBusDisplayIface;

struct _GdmDBusDisplayIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_get_id) (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_remote_hostname) (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_seat_id) (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_session_id) (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_is_initial) (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_is_local) (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusDisplay, g_object_unref)
#endif

GType gdm_dbus_display_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *gdm_dbus_display_interface_info (void);
guint gdm_dbus_display_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void gdm_dbus_display_complete_get_id (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation,
    const gchar *id);

void gdm_dbus_display_complete_get_seat_id (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation,
    const gchar *filename);

void gdm_dbus_display_complete_get_session_id (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation,
    const gchar *session);

void gdm_dbus_display_complete_is_initial (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation,
    gboolean initial);

void gdm_dbus_display_complete_get_remote_hostname (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation,
    const gchar *hostname);

void gdm_dbus_display_complete_is_local (
    GdmDBusDisplay *object,
    GDBusMethodInvocation *invocation,
    gboolean local);



/* D-Bus method calls: */
void gdm_dbus_display_call_get_id (
    GdmDBusDisplay *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_display_call_get_id_finish (
    GdmDBusDisplay *proxy,
    gchar ** out_id,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_display_call_get_id_sync (
    GdmDBusDisplay *proxy,
    gchar ** out_id,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_display_call_get_seat_id (
    GdmDBusDisplay *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_display_call_get_seat_id_finish (
    GdmDBusDisplay *proxy,
    gchar ** out_filename,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_display_call_get_seat_id_sync (
    GdmDBusDisplay *proxy,
    gchar ** out_filename,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_display_call_get_session_id (
    GdmDBusDisplay *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_display_call_get_session_id_finish (
    GdmDBusDisplay *proxy,
    gchar ** out_session,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_display_call_get_session_id_sync (
    GdmDBusDisplay *proxy,
    gchar ** out_session,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_display_call_is_initial (
    GdmDBusDisplay *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_display_call_is_initial_finish (
    GdmDBusDisplay *proxy,
    gboolean* out_initial,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_display_call_is_initial_sync (
    GdmDBusDisplay *proxy,
    gboolean* out_initial,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_display_call_get_remote_hostname (
    GdmDBusDisplay *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_display_call_get_remote_hostname_finish (
    GdmDBusDisplay *proxy,
    gchar ** out_hostname,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_display_call_get_remote_hostname_sync (
    GdmDBusDisplay *proxy,
    gchar ** out_hostname,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_display_call_is_local (
    GdmDBusDisplay *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_display_call_is_local_finish (
    GdmDBusDisplay *proxy,
    gboolean* out_local,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_display_call_is_local_sync (
    GdmDBusDisplay *proxy,
    gboolean* out_local,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define GDM_DBUS_TYPE_DISPLAY_PROXY (gdm_dbus_display_proxy_get_type ())
#define GDM_DBUS_DISPLAY_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_DISPLAY_PROXY, GdmDBusDisplayProxy))
#define GDM_DBUS_DISPLAY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_DISPLAY_PROXY, GdmDBusDisplayProxyClass))
#define GDM_DBUS_DISPLAY_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_DISPLAY_PROXY, GdmDBusDisplayProxyClass))
#define GDM_DBUS_IS_DISPLAY_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_DISPLAY_PROXY))
#define GDM_DBUS_IS_DISPLAY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_DISPLAY_PROXY))

typedef struct _GdmDBusDisplayProxy GdmDBusDisplayProxy;
typedef struct _GdmDBusDisplayProxyClass GdmDBusDisplayProxyClass;
typedef struct _GdmDBusDisplayProxyPrivate GdmDBusDisplayProxyPrivate;

struct _GdmDBusDisplayProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  GdmDBusDisplayProxyPrivate *priv;
};

struct _GdmDBusDisplayProxyClass
{
  GDBusProxyClass parent_class;
};

GType gdm_dbus_display_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusDisplayProxy, g_object_unref)
#endif

void gdm_dbus_display_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusDisplay *gdm_dbus_display_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusDisplay *gdm_dbus_display_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void gdm_dbus_display_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusDisplay *gdm_dbus_display_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusDisplay *gdm_dbus_display_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define GDM_DBUS_TYPE_DISPLAY_SKELETON (gdm_dbus_display_skeleton_get_type ())
#define GDM_DBUS_DISPLAY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_DISPLAY_SKELETON, GdmDBusDisplaySkeleton))
#define GDM_DBUS_DISPLAY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_DISPLAY_SKELETON, GdmDBusDisplaySkeletonClass))
#define GDM_DBUS_DISPLAY_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_DISPLAY_SKELETON, GdmDBusDisplaySkeletonClass))
#define GDM_DBUS_IS_DISPLAY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_DISPLAY_SKELETON))
#define GDM_DBUS_IS_DISPLAY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_DISPLAY_SKELETON))

typedef struct _GdmDBusDisplaySkeleton GdmDBusDisplaySkeleton;
typedef struct _GdmDBusDisplaySkeletonClass GdmDBusDisplaySkeletonClass;
typedef struct _GdmDBusDisplaySkeletonPrivate GdmDBusDisplaySkeletonPrivate;

struct _GdmDBusDisplaySkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  GdmDBusDisplaySkeletonPrivate *priv;
};

struct _GdmDBusDisplaySkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType gdm_dbus_display_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusDisplaySkeleton, g_object_unref)
#endif

GdmDBusDisplay *gdm_dbus_display_skeleton_new (void);


G_END_DECLS

#endif /* __GDM_DISPLAY_GLUE_H__ */
