/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GDM_LOCAL_DISPLAY_FACTORY_GLUE_H__
#define __GDM_LOCAL_DISPLAY_FACTORY_GLUE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.DisplayManager.LocalDisplayFactory */

#define GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY (gdm_dbus_local_display_factory_get_type ())
#define GDM_DBUS_LOCAL_DISPLAY_FACTORY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY, GdmDBusLocalDisplayFactory))
#define GDM_DBUS_IS_LOCAL_DISPLAY_FACTORY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY))
#define GDM_DBUS_LOCAL_DISPLAY_FACTORY_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY, GdmDBusLocalDisplayFactoryIface))

struct _GdmDBusLocalDisplayFactory;
typedef struct _GdmDBusLocalDisplayFactory GdmDBusLocalDisplayFactory;
typedef struct _GdmDBusLocalDisplayFactoryIface GdmDBusLocalDisplayFactoryIface;

struct _GdmDBusLocalDisplayFactoryIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_create_transient_display) (
    GdmDBusLocalDisplayFactory *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_create_user_display) (
    GdmDBusLocalDisplayFactory *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_user);

  gboolean (*handle_destroy_user_display) (
    GdmDBusLocalDisplayFactory *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_user);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusLocalDisplayFactory, g_object_unref)
#endif

GType gdm_dbus_local_display_factory_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *gdm_dbus_local_display_factory_interface_info (void);
guint gdm_dbus_local_display_factory_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void gdm_dbus_local_display_factory_complete_create_transient_display (
    GdmDBusLocalDisplayFactory *object,
    GDBusMethodInvocation *invocation,
    const gchar *id);

void gdm_dbus_local_display_factory_complete_create_user_display (
    GdmDBusLocalDisplayFactory *object,
    GDBusMethodInvocation *invocation);

void gdm_dbus_local_display_factory_complete_destroy_user_display (
    GdmDBusLocalDisplayFactory *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void gdm_dbus_local_display_factory_call_create_transient_display (
    GdmDBusLocalDisplayFactory *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_local_display_factory_call_create_transient_display_finish (
    GdmDBusLocalDisplayFactory *proxy,
    gchar ** out_id,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_local_display_factory_call_create_transient_display_sync (
    GdmDBusLocalDisplayFactory *proxy,
    gchar ** out_id,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_local_display_factory_call_create_user_display (
    GdmDBusLocalDisplayFactory *proxy,
    const gchar *arg_user,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_local_display_factory_call_create_user_display_finish (
    GdmDBusLocalDisplayFactory *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_local_display_factory_call_create_user_display_sync (
    GdmDBusLocalDisplayFactory *proxy,
    const gchar *arg_user,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_local_display_factory_call_destroy_user_display (
    GdmDBusLocalDisplayFactory *proxy,
    const gchar *arg_user,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_local_display_factory_call_destroy_user_display_finish (
    GdmDBusLocalDisplayFactory *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_local_display_factory_call_destroy_user_display_sync (
    GdmDBusLocalDisplayFactory *proxy,
    const gchar *arg_user,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_PROXY (gdm_dbus_local_display_factory_proxy_get_type ())
#define GDM_DBUS_LOCAL_DISPLAY_FACTORY_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_PROXY, GdmDBusLocalDisplayFactoryProxy))
#define GDM_DBUS_LOCAL_DISPLAY_FACTORY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_PROXY, GdmDBusLocalDisplayFactoryProxyClass))
#define GDM_DBUS_LOCAL_DISPLAY_FACTORY_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_PROXY, GdmDBusLocalDisplayFactoryProxyClass))
#define GDM_DBUS_IS_LOCAL_DISPLAY_FACTORY_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_PROXY))
#define GDM_DBUS_IS_LOCAL_DISPLAY_FACTORY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_PROXY))

typedef struct _GdmDBusLocalDisplayFactoryProxy GdmDBusLocalDisplayFactoryProxy;
typedef struct _GdmDBusLocalDisplayFactoryProxyClass GdmDBusLocalDisplayFactoryProxyClass;
typedef struct _GdmDBusLocalDisplayFactoryProxyPrivate GdmDBusLocalDisplayFactoryProxyPrivate;

struct _GdmDBusLocalDisplayFactoryProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  GdmDBusLocalDisplayFactoryProxyPrivate *priv;
};

struct _GdmDBusLocalDisplayFactoryProxyClass
{
  GDBusProxyClass parent_class;
};

GType gdm_dbus_local_display_factory_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusLocalDisplayFactoryProxy, g_object_unref)
#endif

void gdm_dbus_local_display_factory_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusLocalDisplayFactory *gdm_dbus_local_display_factory_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusLocalDisplayFactory *gdm_dbus_local_display_factory_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void gdm_dbus_local_display_factory_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusLocalDisplayFactory *gdm_dbus_local_display_factory_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusLocalDisplayFactory *gdm_dbus_local_display_factory_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_SKELETON (gdm_dbus_local_display_factory_skeleton_get_type ())
#define GDM_DBUS_LOCAL_DISPLAY_FACTORY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_SKELETON, GdmDBusLocalDisplayFactorySkeleton))
#define GDM_DBUS_LOCAL_DISPLAY_FACTORY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_SKELETON, GdmDBusLocalDisplayFactorySkeletonClass))
#define GDM_DBUS_LOCAL_DISPLAY_FACTORY_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_SKELETON, GdmDBusLocalDisplayFactorySkeletonClass))
#define GDM_DBUS_IS_LOCAL_DISPLAY_FACTORY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_SKELETON))
#define GDM_DBUS_IS_LOCAL_DISPLAY_FACTORY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_LOCAL_DISPLAY_FACTORY_SKELETON))

typedef struct _GdmDBusLocalDisplayFactorySkeleton GdmDBusLocalDisplayFactorySkeleton;
typedef struct _GdmDBusLocalDisplayFactorySkeletonClass GdmDBusLocalDisplayFactorySkeletonClass;
typedef struct _GdmDBusLocalDisplayFactorySkeletonPrivate GdmDBusLocalDisplayFactorySkeletonPrivate;

struct _GdmDBusLocalDisplayFactorySkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  GdmDBusLocalDisplayFactorySkeletonPrivate *priv;
};

struct _GdmDBusLocalDisplayFactorySkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType gdm_dbus_local_display_factory_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusLocalDisplayFactorySkeleton, g_object_unref)
#endif

GdmDBusLocalDisplayFactory *gdm_dbus_local_display_factory_skeleton_new (void);


G_END_DECLS

#endif /* __GDM_LOCAL_DISPLAY_FACTORY_GLUE_H__ */
