
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */


#include <glib-object.h>
#include "/build/gdm/src/build/../gdm/daemon/gdm-session.h"
/* enumerations from "/build/gdm/src/build/../gdm/daemon/gdm-session.h" */
GType gdm_session_verification_mode_get_type (void) G_GNUC_CONST;

GType
gdm_session_verification_mode_get_type (void)
{
        static GType etype = 0;

        if (G_UNLIKELY(etype == 0)) {
                static const GEnumValue values[] = {
                { GDM_SESSION_VERIFICATION_MODE_LOGIN, "GDM_SESSION_VERIFICATION_MODE_LOGIN", "login" },
                { GDM_SESSION_VERIFICATION_MODE_REAUTHENTICATE, "GDM_SESSION_VERIFICATION_MODE_REAUTHENTICATE", "reauthenticate" },
                { 0, NULL, NULL }
        };

        etype = g_enum_register_static (g_intern_static_string ("GdmSessionVerificationMode"), values);
    }

    return etype;
}
GType gdm_session_display_mode_get_type (void) G_GNUC_CONST;

GType
gdm_session_display_mode_get_type (void)
{
        static GType etype = 0;

        if (G_UNLIKELY(etype == 0)) {
                static const GEnumValue values[] = {
                { GDM_SESSION_DISPLAY_MODE_NEW_VT, "GDM_SESSION_DISPLAY_MODE_NEW_VT", "new-vt" },
                { GDM_SESSION_DISPLAY_MODE_LOGIND_MANAGED, "GDM_SESSION_DISPLAY_MODE_LOGIND_MANAGED", "logind-managed" },
                { 0, NULL, NULL }
        };

        etype = g_enum_register_static (g_intern_static_string ("GdmSessionDisplayMode"), values);
    }

    return etype;
}
 /**/

/* Generated data ends here */

