/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GDM_LOCAL_DISPLAY_GLUE_H__
#define __GDM_LOCAL_DISPLAY_GLUE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.DisplayManager.LocalDisplay */

#define GDM_DBUS_TYPE_LOCAL_DISPLAY (gdm_dbus_local_display_get_type ())
#define GDM_DBUS_LOCAL_DISPLAY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY, GdmDBusLocalDisplay))
#define GDM_DBUS_IS_LOCAL_DISPLAY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY))
#define GDM_DBUS_LOCAL_DISPLAY_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY, GdmDBusLocalDisplayIface))

struct _GdmDBusLocalDisplay;
typedef struct _GdmDBusLocalDisplay GdmDBusLocalDisplay;
typedef struct _GdmDBusLocalDisplayIface GdmDBusLocalDisplayIface;

struct _GdmDBusLocalDisplayIface
{
  GTypeInterface parent_iface;
};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusLocalDisplay, g_object_unref)
#endif

GType gdm_dbus_local_display_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *gdm_dbus_local_display_interface_info (void);
guint gdm_dbus_local_display_override_properties (GObjectClass *klass, guint property_id_begin);


/* ---- */

#define GDM_DBUS_TYPE_LOCAL_DISPLAY_PROXY (gdm_dbus_local_display_proxy_get_type ())
#define GDM_DBUS_LOCAL_DISPLAY_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_PROXY, GdmDBusLocalDisplayProxy))
#define GDM_DBUS_LOCAL_DISPLAY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_LOCAL_DISPLAY_PROXY, GdmDBusLocalDisplayProxyClass))
#define GDM_DBUS_LOCAL_DISPLAY_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_PROXY, GdmDBusLocalDisplayProxyClass))
#define GDM_DBUS_IS_LOCAL_DISPLAY_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_PROXY))
#define GDM_DBUS_IS_LOCAL_DISPLAY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_LOCAL_DISPLAY_PROXY))

typedef struct _GdmDBusLocalDisplayProxy GdmDBusLocalDisplayProxy;
typedef struct _GdmDBusLocalDisplayProxyClass GdmDBusLocalDisplayProxyClass;
typedef struct _GdmDBusLocalDisplayProxyPrivate GdmDBusLocalDisplayProxyPrivate;

struct _GdmDBusLocalDisplayProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  GdmDBusLocalDisplayProxyPrivate *priv;
};

struct _GdmDBusLocalDisplayProxyClass
{
  GDBusProxyClass parent_class;
};

GType gdm_dbus_local_display_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusLocalDisplayProxy, g_object_unref)
#endif

void gdm_dbus_local_display_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusLocalDisplay *gdm_dbus_local_display_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusLocalDisplay *gdm_dbus_local_display_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void gdm_dbus_local_display_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusLocalDisplay *gdm_dbus_local_display_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusLocalDisplay *gdm_dbus_local_display_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define GDM_DBUS_TYPE_LOCAL_DISPLAY_SKELETON (gdm_dbus_local_display_skeleton_get_type ())
#define GDM_DBUS_LOCAL_DISPLAY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_SKELETON, GdmDBusLocalDisplaySkeleton))
#define GDM_DBUS_LOCAL_DISPLAY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_LOCAL_DISPLAY_SKELETON, GdmDBusLocalDisplaySkeletonClass))
#define GDM_DBUS_LOCAL_DISPLAY_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_SKELETON, GdmDBusLocalDisplaySkeletonClass))
#define GDM_DBUS_IS_LOCAL_DISPLAY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_LOCAL_DISPLAY_SKELETON))
#define GDM_DBUS_IS_LOCAL_DISPLAY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_LOCAL_DISPLAY_SKELETON))

typedef struct _GdmDBusLocalDisplaySkeleton GdmDBusLocalDisplaySkeleton;
typedef struct _GdmDBusLocalDisplaySkeletonClass GdmDBusLocalDisplaySkeletonClass;
typedef struct _GdmDBusLocalDisplaySkeletonPrivate GdmDBusLocalDisplaySkeletonPrivate;

struct _GdmDBusLocalDisplaySkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  GdmDBusLocalDisplaySkeletonPrivate *priv;
};

struct _GdmDBusLocalDisplaySkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType gdm_dbus_local_display_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusLocalDisplaySkeleton, g_object_unref)
#endif

GdmDBusLocalDisplay *gdm_dbus_local_display_skeleton_new (void);


G_END_DECLS

#endif /* __GDM_LOCAL_DISPLAY_GLUE_H__ */
