/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GDM_REMOTE_DISPLAY_GLUE_H__
#define __GDM_REMOTE_DISPLAY_GLUE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.DisplayManager.RemoteDisplay */

#define GDM_DBUS_TYPE_REMOTE_DISPLAY (gdm_dbus_remote_display_get_type ())
#define GDM_DBUS_REMOTE_DISPLAY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_REMOTE_DISPLAY, GdmDBusRemoteDisplay))
#define GDM_DBUS_IS_REMOTE_DISPLAY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_REMOTE_DISPLAY))
#define GDM_DBUS_REMOTE_DISPLAY_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), GDM_DBUS_TYPE_REMOTE_DISPLAY, GdmDBusRemoteDisplayIface))

struct _GdmDBusRemoteDisplay;
typedef struct _GdmDBusRemoteDisplay GdmDBusRemoteDisplay;
typedef struct _GdmDBusRemoteDisplayIface GdmDBusRemoteDisplayIface;

struct _GdmDBusRemoteDisplayIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_set_remote_id) (
    GdmDBusRemoteDisplay *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_remote_id);

  const gchar * (*get_remote_id) (GdmDBusRemoteDisplay *object);

  const gchar * (*get_session_id) (GdmDBusRemoteDisplay *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusRemoteDisplay, g_object_unref)
#endif

GType gdm_dbus_remote_display_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *gdm_dbus_remote_display_interface_info (void);
guint gdm_dbus_remote_display_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void gdm_dbus_remote_display_complete_set_remote_id (
    GdmDBusRemoteDisplay *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void gdm_dbus_remote_display_call_set_remote_id (
    GdmDBusRemoteDisplay *proxy,
    const gchar *arg_remote_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_remote_display_call_set_remote_id_finish (
    GdmDBusRemoteDisplay *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_remote_display_call_set_remote_id_sync (
    GdmDBusRemoteDisplay *proxy,
    const gchar *arg_remote_id,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
const gchar *gdm_dbus_remote_display_get_remote_id (GdmDBusRemoteDisplay *object);
gchar *gdm_dbus_remote_display_dup_remote_id (GdmDBusRemoteDisplay *object);
void gdm_dbus_remote_display_set_remote_id (GdmDBusRemoteDisplay *object, const gchar *value);

const gchar *gdm_dbus_remote_display_get_session_id (GdmDBusRemoteDisplay *object);
gchar *gdm_dbus_remote_display_dup_session_id (GdmDBusRemoteDisplay *object);
void gdm_dbus_remote_display_set_session_id (GdmDBusRemoteDisplay *object, const gchar *value);


/* ---- */

#define GDM_DBUS_TYPE_REMOTE_DISPLAY_PROXY (gdm_dbus_remote_display_proxy_get_type ())
#define GDM_DBUS_REMOTE_DISPLAY_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_REMOTE_DISPLAY_PROXY, GdmDBusRemoteDisplayProxy))
#define GDM_DBUS_REMOTE_DISPLAY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_REMOTE_DISPLAY_PROXY, GdmDBusRemoteDisplayProxyClass))
#define GDM_DBUS_REMOTE_DISPLAY_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_REMOTE_DISPLAY_PROXY, GdmDBusRemoteDisplayProxyClass))
#define GDM_DBUS_IS_REMOTE_DISPLAY_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_REMOTE_DISPLAY_PROXY))
#define GDM_DBUS_IS_REMOTE_DISPLAY_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_REMOTE_DISPLAY_PROXY))

typedef struct _GdmDBusRemoteDisplayProxy GdmDBusRemoteDisplayProxy;
typedef struct _GdmDBusRemoteDisplayProxyClass GdmDBusRemoteDisplayProxyClass;
typedef struct _GdmDBusRemoteDisplayProxyPrivate GdmDBusRemoteDisplayProxyPrivate;

struct _GdmDBusRemoteDisplayProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  GdmDBusRemoteDisplayProxyPrivate *priv;
};

struct _GdmDBusRemoteDisplayProxyClass
{
  GDBusProxyClass parent_class;
};

GType gdm_dbus_remote_display_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusRemoteDisplayProxy, g_object_unref)
#endif

void gdm_dbus_remote_display_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusRemoteDisplay *gdm_dbus_remote_display_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusRemoteDisplay *gdm_dbus_remote_display_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void gdm_dbus_remote_display_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusRemoteDisplay *gdm_dbus_remote_display_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusRemoteDisplay *gdm_dbus_remote_display_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define GDM_DBUS_TYPE_REMOTE_DISPLAY_SKELETON (gdm_dbus_remote_display_skeleton_get_type ())
#define GDM_DBUS_REMOTE_DISPLAY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_REMOTE_DISPLAY_SKELETON, GdmDBusRemoteDisplaySkeleton))
#define GDM_DBUS_REMOTE_DISPLAY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_REMOTE_DISPLAY_SKELETON, GdmDBusRemoteDisplaySkeletonClass))
#define GDM_DBUS_REMOTE_DISPLAY_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_REMOTE_DISPLAY_SKELETON, GdmDBusRemoteDisplaySkeletonClass))
#define GDM_DBUS_IS_REMOTE_DISPLAY_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_REMOTE_DISPLAY_SKELETON))
#define GDM_DBUS_IS_REMOTE_DISPLAY_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_REMOTE_DISPLAY_SKELETON))

typedef struct _GdmDBusRemoteDisplaySkeleton GdmDBusRemoteDisplaySkeleton;
typedef struct _GdmDBusRemoteDisplaySkeletonClass GdmDBusRemoteDisplaySkeletonClass;
typedef struct _GdmDBusRemoteDisplaySkeletonPrivate GdmDBusRemoteDisplaySkeletonPrivate;

struct _GdmDBusRemoteDisplaySkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  GdmDBusRemoteDisplaySkeletonPrivate *priv;
};

struct _GdmDBusRemoteDisplaySkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType gdm_dbus_remote_display_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusRemoteDisplaySkeleton, g_object_unref)
#endif

GdmDBusRemoteDisplay *gdm_dbus_remote_display_skeleton_new (void);


G_END_DECLS

#endif /* __GDM_REMOTE_DISPLAY_GLUE_H__ */
