/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GDM_SESSION_WORKER_GLUE_H__
#define __GDM_SESSION_WORKER_GLUE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.DisplayManager.Worker */

#define GDM_DBUS_TYPE_WORKER (gdm_dbus_worker_get_type ())
#define GDM_DBUS_WORKER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_WORKER, GdmDBusWorker))
#define GDM_DBUS_IS_WORKER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_WORKER))
#define GDM_DBUS_WORKER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), GDM_DBUS_TYPE_WORKER, GdmDBusWorkerIface))

struct _GdmDBusWorker;
typedef struct _GdmDBusWorker GdmDBusWorker;
typedef struct _GdmDBusWorkerIface GdmDBusWorkerIface;

struct _GdmDBusWorkerIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_authenticate) (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_authorize) (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_establish_credentials) (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_initialize) (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_details);

  gboolean (*handle_open) (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_set_environment_variable) (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_name,
    const gchar *arg_value);

  gboolean (*handle_set_language_name) (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_language);

  gboolean (*handle_set_session_display_mode) (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_mode);

  gboolean (*handle_set_session_name) (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_session_name);

  gboolean (*handle_start_program) (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_command);

  gboolean (*handle_start_reauthentication) (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation,
    gint arg_pid_of_caller,
    gint arg_uid_of_caller);

  void (*cancel_pending_query) (
    GdmDBusWorker *object);

  void (*reauthenticated) (
    GdmDBusWorker *object,
    const gchar *arg_service_name,
    gint arg_pid_of_caller);

  void (*saved_language_name_read) (
    GdmDBusWorker *object,
    const gchar *arg_language_name);

  void (*saved_session_name_read) (
    GdmDBusWorker *object,
    const gchar *arg_session_name);

  void (*saved_session_type_read) (
    GdmDBusWorker *object,
    const gchar *arg_session_type);

  void (*session_exited) (
    GdmDBusWorker *object,
    const gchar *arg_service_name,
    gint arg_status);

  void (*username_changed) (
    GdmDBusWorker *object,
    const gchar *arg_new_username);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusWorker, g_object_unref)
#endif

GType gdm_dbus_worker_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *gdm_dbus_worker_interface_info (void);
guint gdm_dbus_worker_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void gdm_dbus_worker_complete_authenticate (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

void gdm_dbus_worker_complete_authorize (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

void gdm_dbus_worker_complete_establish_credentials (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

void gdm_dbus_worker_complete_open (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation,
    const gchar *session_id);

void gdm_dbus_worker_complete_set_language_name (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

void gdm_dbus_worker_complete_set_session_name (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

void gdm_dbus_worker_complete_set_session_display_mode (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

void gdm_dbus_worker_complete_set_environment_variable (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

void gdm_dbus_worker_complete_start_program (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation,
    gint child_pid);

void gdm_dbus_worker_complete_initialize (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation);

void gdm_dbus_worker_complete_start_reauthentication (
    GdmDBusWorker *object,
    GDBusMethodInvocation *invocation,
    const gchar *address);



/* D-Bus signal emissions functions: */
void gdm_dbus_worker_emit_session_exited (
    GdmDBusWorker *object,
    const gchar *arg_service_name,
    gint arg_status);

void gdm_dbus_worker_emit_saved_language_name_read (
    GdmDBusWorker *object,
    const gchar *arg_language_name);

void gdm_dbus_worker_emit_saved_session_name_read (
    GdmDBusWorker *object,
    const gchar *arg_session_name);

void gdm_dbus_worker_emit_saved_session_type_read (
    GdmDBusWorker *object,
    const gchar *arg_session_type);

void gdm_dbus_worker_emit_username_changed (
    GdmDBusWorker *object,
    const gchar *arg_new_username);

void gdm_dbus_worker_emit_reauthenticated (
    GdmDBusWorker *object,
    const gchar *arg_service_name,
    gint arg_pid_of_caller);

void gdm_dbus_worker_emit_cancel_pending_query (
    GdmDBusWorker *object);



/* D-Bus method calls: */
void gdm_dbus_worker_call_authenticate (
    GdmDBusWorker *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_worker_call_authenticate_finish (
    GdmDBusWorker *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_worker_call_authenticate_sync (
    GdmDBusWorker *proxy,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_worker_call_authorize (
    GdmDBusWorker *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_worker_call_authorize_finish (
    GdmDBusWorker *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_worker_call_authorize_sync (
    GdmDBusWorker *proxy,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_worker_call_establish_credentials (
    GdmDBusWorker *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_worker_call_establish_credentials_finish (
    GdmDBusWorker *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_worker_call_establish_credentials_sync (
    GdmDBusWorker *proxy,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_worker_call_open (
    GdmDBusWorker *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_worker_call_open_finish (
    GdmDBusWorker *proxy,
    gchar ** out_session_id,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_worker_call_open_sync (
    GdmDBusWorker *proxy,
    gchar ** out_session_id,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_worker_call_set_language_name (
    GdmDBusWorker *proxy,
    const gchar *arg_language,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_worker_call_set_language_name_finish (
    GdmDBusWorker *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_worker_call_set_language_name_sync (
    GdmDBusWorker *proxy,
    const gchar *arg_language,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_worker_call_set_session_name (
    GdmDBusWorker *proxy,
    const gchar *arg_session_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_worker_call_set_session_name_finish (
    GdmDBusWorker *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_worker_call_set_session_name_sync (
    GdmDBusWorker *proxy,
    const gchar *arg_session_name,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_worker_call_set_session_display_mode (
    GdmDBusWorker *proxy,
    const gchar *arg_mode,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_worker_call_set_session_display_mode_finish (
    GdmDBusWorker *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_worker_call_set_session_display_mode_sync (
    GdmDBusWorker *proxy,
    const gchar *arg_mode,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_worker_call_set_environment_variable (
    GdmDBusWorker *proxy,
    const gchar *arg_name,
    const gchar *arg_value,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_worker_call_set_environment_variable_finish (
    GdmDBusWorker *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_worker_call_set_environment_variable_sync (
    GdmDBusWorker *proxy,
    const gchar *arg_name,
    const gchar *arg_value,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_worker_call_start_program (
    GdmDBusWorker *proxy,
    const gchar *arg_command,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_worker_call_start_program_finish (
    GdmDBusWorker *proxy,
    gint* out_child_pid,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_worker_call_start_program_sync (
    GdmDBusWorker *proxy,
    const gchar *arg_command,
    gint* out_child_pid,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_worker_call_initialize (
    GdmDBusWorker *proxy,
    GVariant *arg_details,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_worker_call_initialize_finish (
    GdmDBusWorker *proxy,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_worker_call_initialize_sync (
    GdmDBusWorker *proxy,
    GVariant *arg_details,
    GCancellable *cancellable,
    GError **error);

void gdm_dbus_worker_call_start_reauthentication (
    GdmDBusWorker *proxy,
    gint arg_pid_of_caller,
    gint arg_uid_of_caller,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean gdm_dbus_worker_call_start_reauthentication_finish (
    GdmDBusWorker *proxy,
    gchar ** out_address,
    GAsyncResult *res,
    GError **error);

gboolean gdm_dbus_worker_call_start_reauthentication_sync (
    GdmDBusWorker *proxy,
    gint arg_pid_of_caller,
    gint arg_uid_of_caller,
    gchar ** out_address,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define GDM_DBUS_TYPE_WORKER_PROXY (gdm_dbus_worker_proxy_get_type ())
#define GDM_DBUS_WORKER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_WORKER_PROXY, GdmDBusWorkerProxy))
#define GDM_DBUS_WORKER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_WORKER_PROXY, GdmDBusWorkerProxyClass))
#define GDM_DBUS_WORKER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_WORKER_PROXY, GdmDBusWorkerProxyClass))
#define GDM_DBUS_IS_WORKER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_WORKER_PROXY))
#define GDM_DBUS_IS_WORKER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_WORKER_PROXY))

typedef struct _GdmDBusWorkerProxy GdmDBusWorkerProxy;
typedef struct _GdmDBusWorkerProxyClass GdmDBusWorkerProxyClass;
typedef struct _GdmDBusWorkerProxyPrivate GdmDBusWorkerProxyPrivate;

struct _GdmDBusWorkerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  GdmDBusWorkerProxyPrivate *priv;
};

struct _GdmDBusWorkerProxyClass
{
  GDBusProxyClass parent_class;
};

GType gdm_dbus_worker_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusWorkerProxy, g_object_unref)
#endif

void gdm_dbus_worker_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusWorker *gdm_dbus_worker_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusWorker *gdm_dbus_worker_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void gdm_dbus_worker_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
GdmDBusWorker *gdm_dbus_worker_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GdmDBusWorker *gdm_dbus_worker_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define GDM_DBUS_TYPE_WORKER_SKELETON (gdm_dbus_worker_skeleton_get_type ())
#define GDM_DBUS_WORKER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GDM_DBUS_TYPE_WORKER_SKELETON, GdmDBusWorkerSkeleton))
#define GDM_DBUS_WORKER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GDM_DBUS_TYPE_WORKER_SKELETON, GdmDBusWorkerSkeletonClass))
#define GDM_DBUS_WORKER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDM_DBUS_TYPE_WORKER_SKELETON, GdmDBusWorkerSkeletonClass))
#define GDM_DBUS_IS_WORKER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDM_DBUS_TYPE_WORKER_SKELETON))
#define GDM_DBUS_IS_WORKER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GDM_DBUS_TYPE_WORKER_SKELETON))

typedef struct _GdmDBusWorkerSkeleton GdmDBusWorkerSkeleton;
typedef struct _GdmDBusWorkerSkeletonClass GdmDBusWorkerSkeletonClass;
typedef struct _GdmDBusWorkerSkeletonPrivate GdmDBusWorkerSkeletonPrivate;

struct _GdmDBusWorkerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  GdmDBusWorkerSkeletonPrivate *priv;
};

struct _GdmDBusWorkerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType gdm_dbus_worker_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GdmDBusWorkerSkeleton, g_object_unref)
#endif

GdmDBusWorker *gdm_dbus_worker_skeleton_new (void);


G_END_DECLS

#endif /* __GDM_SESSION_WORKER_GLUE_H__ */
